/* Sistemas Integrados TEKNISA
   Planejamento Alimentao Industrial

Nome: 7080_PLASTPO_CONSUMO_REQUISICAO_EXTRA.SQL

Descrio:
Gera requisio extra de matria prima automaticamente, com base no consumo do
crdapio, consumo fixo ou consumo per-capita de determinado servio, gravando
na tabela PREVRETI.
Obs.: O nmero de comensais utilizado no  o do cardpio e sim o parametrizado
      na tela PLA00910, recebido como parmetro na STP (P_NRCOMENSAIS).

P_CDFILIAL    - Cdigo da Filial
P_CDSERVFILI  - Cdigo do Servio que ser gerada a Requisio Extra
P_DTCARDAPIO  - Data da Requisio
P_NRREQUEXTR  - Nmero da Requisio (poder utilizar um j existente, inserido
                   pelo usurio, ou um sequencial gerado pela tela PLA00910)
P_NRCOMENSAIS - Nmero de Comensais (usados no clculo de consumo do cardpio e
                   consumo per-capita)
P_CDOPERADOR  - Cdigo do Operador
P_IDTIPOCONS  - Identificador do dia da semana ('8' se feriado)

Criao:
24/07/2007 Eurico Jos Faria Klein
*/

CREATE OR REPLACE
PROCEDURE CONSUMO_REQUISICAO_EXTRA
  (P_CDFILIAL    IN VARCHAR2,
   P_CDSERVFILI  IN VARCHAR2,
   P_DTCARDAPIO  IN DATE,
   P_NRREQUEXTR  IN VARCHAR2,
   P_NRCOMENSAIS IN FLOAT,
   P_CDOPERADOR  IN VARCHAR2,
   P_IDTIPOCONS  IN VARCHAR2) IS

   /*VARIVEIS*/
   V_NRPRPREVRETI INTEGER := 1;
   V_HRINICATEN   VARCHAR2(4);
   V_NRSEQUENCIAL VARCHAR2(4);
BEGIN
	FOR C_CARDAPIO IN (
      SELECT PR.CDPRODUTO,
             SUM(PR.QTBRUTA) AS QTBRUTA,
             PR.IDQTDECONS,
             DECODE(SE.IDUTRETIPRAT, 'S' , PR.CDTIPOPRAT  , NULL) AS CDTIPOPRAT,
             DECODE(SE.IDUTRETIPRAT, 'S' , PR.CDPRATO     , NULL) AS CDPRATO,
             DECODE(SE.IDUTRETIPRAT, 'S' , PR.CDFILAUXPRAT, NULL) AS CDFILAUXPRAT,
             SE.IDUTRETIPRAT AS IDPREVTPPRAT
      FROM(
           SELECT R.CDPRODUTO,
                  R.QTBRUTASE * P_NRCOMENSAIS AS QTBRUTA,
                  'C' IDQTDECONS,
                  P.CDTIPOPRAT,
                  P.CDPRATO,
                  P.CDFILAUXPRAT
           FROM CARDADIA C, COMPCARD P, RECESERV R
           WHERE (C.CDFILIAL     = P_CDFILIAL    )
             AND (C.CDSERVFILI   = P_CDSERVFILI  )
             AND (C.DTCARDAPIO   = P_DTCARDAPIO  )
             AND (C.CDFILIAL     = P.CDFILIAL    )
             AND (C.NRCARDAPIO   = P.NRCARDAPIO  )
             AND (R.CDPRATO      = P.CDPRATO     )
             AND (R.CDFILAUXPRAT = P.CDFILAUXPRAT)
             AND (R.CDFILIAL     = C.CDFILIAL    )
             AND (R.CDSERVFILI   = C.CDSERVFILI  )
          UNION ALL
           SELECT R.CDPRODUTO,
                  R.QTBRUTALO * P_NRCOMENSAIS AS QTBRUTA,
                  'C' IDQTDECONS,
                  P.CDTIPOPRAT,
                  P.CDPRATO,
                  P.CDFILAUXPRAT
           FROM CARDADIA C, COMPCARD P, RECEFILI R
           WHERE (C.CDFILIAL     = P_CDFILIAL    )
             AND (C.CDSERVFILI   = P_CDSERVFILI  )
             AND (C.DTCARDAPIO   = P_DTCARDAPIO  )
             AND (C.CDFILIAL     = P.CDFILIAL    )
             AND (C.NRCARDAPIO   = P.NRCARDAPIO  )
             AND (R.CDPRATO      = P.CDPRATO     )
             AND (R.CDFILAUXPRAT = P.CDFILAUXPRAT)
             AND (R.CDFILIAL     = C.CDFILIAL    )
             AND NOT EXISTS (SELECT RECESERV.CDPRATO
                             FROM RECESERV
                             WHERE (RECESERV.CDFILIAL     = C.CDFILIAL    )
                               AND (RECESERV.CDSERVFILI   = C.CDSERVFILI  )
                               AND (RECESERV.CDPRATO      = P.CDPRATO     )
                               AND (RECESERV.CDFILAUXPRAT = P.CDFILAUXPRAT))
          UNION ALL
           SELECT R.CDPRODUTO,
                  R.QTBRUTAPR * P_NRCOMENSAIS AS QTBRUTA,
                  'C' IDQTDECONS,
                  P.CDTIPOPRAT,
                  P.CDPRATO,
                  P.CDFILAUXPRAT
           FROM CARDADIA C, COMPCARD P, RECEPADR R
           WHERE (C.CDFILIAL     = P_CDFILIAL    )
             AND (C.CDSERVFILI   = P_CDSERVFILI  )
             AND (C.DTCARDAPIO   = P_DTCARDAPIO  )
             AND (C.CDFILIAL     = P.CDFILIAL    )
             AND (C.NRCARDAPIO   = P.NRCARDAPIO  )
             AND (R.CDPRATO      = P.CDPRATO     )
             AND (R.CDFILAUXPRAT = P.CDFILAUXPRAT)
             AND NOT EXISTS (SELECT RECESERV.CDPRATO
                             FROM RECESERV
                             WHERE (RECESERV.CDFILIAL     = C.CDFILIAL    )
                               AND (RECESERV.CDSERVFILI   = C.CDSERVFILI  )
                               AND (RECESERV.CDPRATO      = P.CDPRATO     )
                               AND (RECESERV.CDFILAUXPRAT = P.CDFILAUXPRAT))
             AND NOT EXISTS (SELECT RECEFILI.CDPRATO
                             FROM RECEFILI
                             WHERE (RECEFILI.CDFILIAL     = C.CDFILIAL    )
                               AND (RECEFILI.CDPRATO      = P.CDPRATO     )
                               AND (RECEFILI.CDFILAUXPRAT = P.CDFILAUXPRAT))
          UNION ALL
           SELECT C.CDPRODUTO,
                  DECODE(C.IDQTDECONS, 'P', C.QTCONSSERV * P_NRCOMENSAIS, C.QTCONSSERV) AS QTBRUTA,
                  C.IDQTDECONS,
                  DECODE(S.IDUTRETIPRAT, 'S',
                         DECODE(C.IDQTDECONS, 'P', P.CDTPRCONSPER, P.CDTPRCONSFIX),
                         NULL) AS CDTIPOPRAT,
                  DECODE(S.IDUTRETIPRAT, 'S',
                         DECODE(C.IDQTDECONS, 'P', P.CDPRACONSPER, P.CDPRACONSFIX),
                         NULL) AS CDPRATO,
                  DECODE(S.IDUTRETIPRAT, 'S',
                         DECODE(C.IDQTDECONS, 'P', P.CDFILCONSPER, P.CDFILCONSFIX),
                         NULL) AS CDFILAUXPRAT
           FROM CONSSERV C, PARAMPLANEJA P, SERVICO S
           WHERE (C.CDFILIAL   = P_CDFILIAL  )
             AND (C.CDSERVFILI = P_CDSERVFILI)
             AND (C.IDTIPOCONS = P_IDTIPOCONS)
             AND (C.CDFILIAL   = P.CDFILIAL  )
             AND (C.CDFILIAL   = S.CDFILIAL  )
             AND (C.CDSERVFILI = S.CDSERVFILI)
             AND (C.DTCONSSERV IN (SELECT MAX(CONSSERV.DTCONSSERV)
                                   FROM CONSSERV
                                   WHERE (CONSSERV.CDFILIAL    = P_CDFILIAL  )
                                     AND (CONSSERV.CDSERVFILI  = P_CDSERVFILI)
                                     AND (CONSSERV.DTCONSSERV <= P_DTCARDAPIO)
                                     AND (CONSSERV.IDTIPOCONS <> '9'         )))
         ) PR, SERVICO SE
      WHERE (SE.CDFILIAL   = P_CDFILIAL  )
        AND (SE.CDSERVFILI = P_CDSERVFILI)
      GROUP BY PR.CDPRODUTO,
               PR.IDQTDECONS,
               DECODE(SE.IDUTRETIPRAT, 'S', PR.CDTIPOPRAT  , NULL),
               DECODE(SE.IDUTRETIPRAT, 'S', PR.CDPRATO     , NULL),
               DECODE(SE.IDUTRETIPRAT, 'S', PR.CDFILAUXPRAT, NULL),
               SE.IDUTRETIPRAT
   )LOOP

      SELECT HRINICATEN
      INTO V_HRINICATEN
      FROM SERVICO
      WHERE (CDFILIAL   = P_CDFILIAL)
        AND (CDSERVFILI = P_CDSERVFILI)
        AND (IDTIPOSERV <> 'A')
        AND ((IDSERVATIV <> 'N') OR (IDSERVATIV IS NULL));

      /*ESTE DELETE EXCLUI TODO ITEM DA REQUISIO QUE ESTEJA PARAMETRIZADO
        DIFERENTE DA CONFIGURAO ATUAL, EVITANDO, NA MESMA REQUISIO, PRODUTOS
        QUE UTILIZAM E QUE NO UTILIZAM RETIRADAS POR PRATO.*/

      DELETE FROM PREVRETI
      WHERE (CDFILIAL      = P_CDFILIAL             )
        AND (CDSERVFILI    = P_CDSERVFILI           )
        AND (DTPREVRETI    = P_DTCARDAPIO           )
        AND (NRREQUEXTR    = P_NRREQUEXTR           )
        AND (IDPREVTPPRAT <> (SELECT IDUTRETIPRAT
                              FROM SERVICO
                              WHERE SERVICO.CDFILIAL   = PREVRETI.CDFILIAL
                                AND SERVICO.CDSERVFILI = PREVRETI.CDSERVFILI));

      /*A VARIVEL V_NRSEQUENCIAL RECEBE O CAMPO NRPRPREVRETI PARA VERIFICAR
        SE H O MESMO PRODUTO NA REQUISICAO E, AO INVS DE INSERIR O PRODUTO
        NOVAMENTE, ATUALIZA-SE O REGISTRO. SE A VARIVEL ESTIVER NULA, SIGNIFICA
        QUE O PRODUTO NO EST NA REQUISIO E APLICA-SE O INSERT NORMALMENTE.
        A EXPRESSAO C_CARDAPIO.IDPREVTPPRAT = 'N' FOI UTILIZADA NO IF PARA EVITAR
        A COMPARAO NULL = NULL NO WHERE, NOS CAMPOS CDPRATO, CDTIPOPRAT E
        CDFILAUXPRAT.*/

      IF (C_CARDAPIO.IDPREVTPPRAT = 'N') THEN
         SELECT MAX(NRPRPREVRETI)
         INTO V_NRSEQUENCIAL
         FROM PREVRETI
         WHERE (CDFILIAL     = P_CDFILIAL           )
           AND (CDSERVFILI   = P_CDSERVFILI         )
           AND (DTPREVRETI   = P_DTCARDAPIO         )
           AND (NRREQUEXTR   = P_NRREQUEXTR         )
           AND (IDPREVRETI   = C_CARDAPIO.IDQTDECONS)
           AND (CDPRODUTO    = C_CARDAPIO.CDPRODUTO )
           AND (DTLIMITERETI = P_DTCARDAPIO         )
           AND (HRLIMITERETI = V_HRINICATEN         );
      ELSE
         SELECT MAX(NRPRPREVRETI)
         INTO V_NRSEQUENCIAL
         FROM PREVRETI
         WHERE (CDFILIAL     = P_CDFILIAL             )
           AND (CDSERVFILI   = P_CDSERVFILI           )
           AND (DTPREVRETI   = P_DTCARDAPIO           )
           AND (NRREQUEXTR   = P_NRREQUEXTR           )
           AND (IDPREVRETI   = C_CARDAPIO.IDQTDECONS  )
           AND (CDPRODUTO    = C_CARDAPIO.CDPRODUTO   )
           AND (DTLIMITERETI = P_DTCARDAPIO           )
           AND (HRLIMITERETI = V_HRINICATEN           )
           AND (CDPRATO      = C_CARDAPIO.CDPRATO     )
           AND (CDTIPOPRAT   = C_CARDAPIO.CDTIPOPRAT  )
           AND (CDFILAUXPRAT = C_CARDAPIO.CDFILAUXPRAT);
      END IF;

      IF V_NRSEQUENCIAL IS NULL THEN
         INSERT INTO PREVRETI
             (CDFILIAL     , CDSERVFILI, DTPREVRETI  , NRREQUEXTR  ,
              IDPREVRETI   , CDPRODUTO , DTLIMITERETI, HRLIMITERETI,
              NRPRPREVRETI , QTPREVRETI, CDTIPOPRAT  , CDPRATO     ,
              CDFILAUXPRAT , CDOPERADOR, IDPREVTPPRAT)
         VALUES
             (P_CDFILIAL                  , P_CDSERVFILI        , P_DTCARDAPIO           , P_NRREQUEXTR      ,
              C_CARDAPIO.IDQTDECONS       , C_CARDAPIO.CDPRODUTO, P_DTCARDAPIO           , V_HRINICATEN      ,
              LPAD(V_NRPRPREVRETI, 4, '0'), C_CARDAPIO.QTBRUTA  , C_CARDAPIO.CDTIPOPRAT  , C_CARDAPIO.CDPRATO,
              C_CARDAPIO.CDFILAUXPRAT     , P_CDOPERADOR        , C_CARDAPIO.IDPREVTPPRAT);

         V_NRPRPREVRETI := V_NRPRPREVRETI + 1;
      ELSE
         IF (C_CARDAPIO.IDPREVTPPRAT = 'N') THEN
            UPDATE PREVRETI
            SET QTPREVRETI = C_CARDAPIO.QTBRUTA,
                QTPRERETT1 = 0,
                QTPRERETT2 = 0,
                QTPRERETT3 = 0,
                QTPRERETT4 = 0,
                CDSETORCOZ = NULL
            WHERE (CDFILIAL     = P_CDFILIAL             )
              AND (CDSERVFILI   = P_CDSERVFILI           )
              AND (DTPREVRETI   = P_DTCARDAPIO           )
              AND (NRREQUEXTR   = P_NRREQUEXTR           )
              AND (IDPREVRETI   = C_CARDAPIO.IDQTDECONS  )
              AND (CDPRODUTO    = C_CARDAPIO.CDPRODUTO   )
              AND (DTLIMITERETI = P_DTCARDAPIO           )
              AND (HRLIMITERETI = V_HRINICATEN           )
              AND (NRPRPREVRETI = V_NRSEQUENCIAL         );
         ELSE
            UPDATE PREVRETI
            SET QTPREVRETI = C_CARDAPIO.QTBRUTA,
                QTPRERETT1 = 0,
                QTPRERETT2 = 0,
                QTPRERETT3 = 0,
                QTPRERETT4 = 0,
                CDSETORCOZ = NULL
            WHERE (CDFILIAL     = P_CDFILIAL             )
              AND (CDSERVFILI   = P_CDSERVFILI           )
              AND (DTPREVRETI   = P_DTCARDAPIO           )
              AND (NRREQUEXTR   = P_NRREQUEXTR           )
              AND (IDPREVRETI   = C_CARDAPIO.IDQTDECONS  )
              AND (CDPRODUTO    = C_CARDAPIO.CDPRODUTO   )
              AND (DTLIMITERETI = P_DTCARDAPIO           )
              AND (HRLIMITERETI = V_HRINICATEN           )
              AND (NRPRPREVRETI = V_NRSEQUENCIAL         )
              AND (CDPRATO      = C_CARDAPIO.CDPRATO     )
              AND (CDTIPOPRAT   = C_CARDAPIO.CDTIPOPRAT  )
              AND (CDFILAUXPRAT = C_CARDAPIO.CDFILAUXPRAT);
         END IF;
      END IF;

   END LOOP;
   COMMIT;
END;
/



-- VERSAO --
BEGIN ATUVEROBJ('CONSUMO_REQUISICAO_EXTRA','4.11.157'); END;
/

